#define DEBUG true  // set to true for debug output to serial, false for no debug ouput
#define DEBUG_SERIAL if(DEBUG)Serial

#include <BitBang_I2C.h>

#define SDA_PIN   18 // use external 22k resistor to 3V3
#define SCL_PIN   19 // use external 22k resistor to 3V3
#define RESET_PIN 20 // use external 10k resistor to GND
#define SMUTE_PIN 21 // software mute pin
#define HMUTE_PIN 22 // hardware mute pin

byte mute_mode                   = 3;  // 0 = never mute, 1 = software mute, 2 = hardware mute, 3 = both software and hardware mute
byte reset_pulses                = 3;  // how many reset pulses will be issued
byte command_repeats             = 12; // how many times the command set will be sent

byte input_frame_sync_polarity   = 0;  // 0 = LRCI low means left channel, 1 = LRCI high means left channel
byte input_slot_length           = 2;  // 0 = 16bit clocks, 1 = 20bit clocks, 2 = 24bit clocks, 3 = 32bit clocks
byte system_clock_multiple       = 1;  // 0 = 256x 4.096-12.8Mhz, 1 = 384x 6.1440-19.2Mhz, 2 = 512x 8.192-25.6Mhz, 3 = 768x 12.288-38.4Mhz
byte input_data_justification    = 1;  // 0 = left justified, 1 = right justified
byte input_word_size             = 0;  // 0 = 16bit, 1 = 18bit (not available in right justified modes), 2 = 20bit, 3 = 24bit
byte input_I2S_mode              = 0;  // 0 = data on word clock, 1 = data 1bit after word clock
byte input_sampling_frequency    = 0;  // 0 = 44.1kHz, 1 = 48kHz, 2 = 88.2kHz, 3 = 96kHz, 4 = 176.4kHz, 5= 192kHz
byte input_bit_clock_polarity    = 1;  // 0 = falling edge, 1 = rising edge

byte output_bit_clock_polarity   = 1;  // 0 = falling edge, 1 = rising edge
byte oversampling_rate           = 3;  // 1 = 2x oversampling, 2 = 4x oversampling, 3 = 8x oversampling
byte output_word_clock_polarity  = 1;  // 0 = high to low at the end of the output word, 1 = low to high at the end of the output word
byte gain_scale_enable           = 1;  // 0 = digital attenuation 6dB internally, 1 = 6dB external signaled on GAIN pin
byte output_audio_polarity       = 0;  // 0 = non inverted, 1 = inverted
byte output_word_size            = 3;  // 0 = 16bit, 1 = 20bit, 2 = 24bit, 3 = 32bit
byte output_multiplex            = 0;  // 0 = output to DOL and DOR, 1 = left and right outputs interleaved on DOL
byte dither_mode                 = 2;  // 0 to 7 = dither mode 0 to 7
byte output_COB                  = 1;  // 0 = COB, 1 = 2s compliment

// DO NOT CHANGE SETTINGS BELOW THIS LINE !

byte set_input_configuration     = 0;  // do not change!
byte set_output_configuration    = 1;  // do not change!
byte set_attenuator              = 2;  // do not change!

BBI2C bbi2c;
uint32_t register_0, register_1, register_2;
int i;

void setup() {
  delay(2000);
  DEBUG_SERIAL.begin(115200);

  pinMode (HMUTE_PIN, OUTPUT);
  digitalWrite(HMUTE_PIN, LOW);
  pinMode (SMUTE_PIN, OUTPUT);
  digitalWrite(SMUTE_PIN, LOW);
  pinMode (RESET_PIN, OUTPUT);
  digitalWrite(RESET_PIN, HIGH);
  delay(100);
  pulse_reset(reset_pulses);

  delay(1000);
  bbi2c.bWire = 0; // use bit bang, not wire library
  bbi2c.iSDA = SDA_PIN;
  bbi2c.iSCL = SCL_PIN;
  I2CInit(&bbi2c, 15000L);
  delay(3);

  register_prepare();
  send_commands();
}

void pulse_reset(int times) {
  DEBUG_SERIAL.println("reseting, please wait...");
  for (int k = 1; k <= times; k++) {digitalWrite(RESET_PIN, LOW); delay(1); digitalWrite(RESET_PIN, HIGH); delay(50);}
  DEBUG_SERIAL.print(times);
  DEBUG_SERIAL.println(" reset pulses issued");
}

void register_prepare() {
  register_0 = 0;
  register_0 |= uint32_t(set_input_configuration) << 20;
  register_0 |= uint32_t(input_frame_sync_polarity) << 12;
  register_0 |= uint32_t(input_slot_length) << 10;
  register_0 |= uint32_t(system_clock_multiple) << 8;
  register_0 |= uint32_t(input_data_justification) << 7;
  register_0 |= uint32_t(input_word_size) << 5;
  register_0 |= uint32_t(input_I2S_mode) << 4;
  register_0 |= uint32_t(input_sampling_frequency) << 1;
  register_0 |= uint32_t(input_bit_clock_polarity);

  register_1 = 0;
  register_1 |= uint32_t(set_output_configuration) << 20;
  register_1 |= uint32_t(output_bit_clock_polarity) << 12;
  register_1 |= uint32_t(oversampling_rate) << 10;
  register_1 |= uint32_t(output_word_clock_polarity) << 9;
  register_1 |= uint32_t(gain_scale_enable) << 8;
  register_1 |= uint32_t(output_audio_polarity) << 7;
  register_1 |= uint32_t(output_word_size) << 5;
  register_1 |= uint32_t(output_multiplex) << 4;
  register_1 |= uint32_t(dither_mode) << 1;
  register_1 |= uint32_t(output_COB);

  register_2 = 0;
  register_2 |= uint32_t(set_attenuator) << 20;
}

void send_commands() {
  mute(1);
  delay(10);
  for (i = 1; i <= command_repeats; i++) {
    i2c_transfer(register_0);
    i2c_transfer(register_1);
    i2c_transfer(register_2);
    delay(3);
  }
  delay(10);
  mute(0);
}

void mute(bool muted) {
  if (muted && mute_mode != 0) {
    if (mute_mode == 1 || mute_mode == 3) {
      DEBUG_SERIAL.println("software mute on");
      digitalWrite(SMUTE_PIN, muted);
      delay(1000);
    }
    if (mute_mode == 2 || mute_mode == 3) {
      DEBUG_SERIAL.println("hardware mute on");
      digitalWrite(HMUTE_PIN, muted);
      delay(100);
    }
  }
  else if (mute_mode != 0) {
    if (mute_mode == 2 || mute_mode == 3) {
      DEBUG_SERIAL.println("hardware mute off");
      digitalWrite(HMUTE_PIN, muted);
      delay(100);
    }
    if (mute_mode == 1 || mute_mode == 3) {
      DEBUG_SERIAL.println("software mute off");
      digitalWrite(SMUTE_PIN, muted);
      delay(1000);
    }
  }
}

void i2c_transfer(uint32_t value) {
  byte buf[3];
  buf[0] = (value >> 16) & 0xFF;
  buf[1] = (value >> 8) & 0xFF;
  buf[2] = value & 0xFF;
  bool success = 0;
  while (!success) {
    DEBUG_SERIAL.print(i);
    DEBUG_SERIAL.print(" send BIN: "); for (char j = 23; j >= 0; j--) {DEBUG_SERIAL.print(bitRead(value, j));}
    DEBUG_SERIAL.print(" HEX: ");
    DEBUG_SERIAL.print(buf[0], HEX);
    DEBUG_SERIAL.print(" ");
    DEBUG_SERIAL.print(buf[1], HEX);
    DEBUG_SERIAL.print(" ");
    DEBUG_SERIAL.print(buf[2], HEX);
    int bytes_written = I2CWrite(&bbi2c, 0b0000000, buf, sizeof buf);
    if (bytes_written == 0) {
      DEBUG_SERIAL.println(" error!");
    } else {
      DEBUG_SERIAL.print(" success: ");
      DEBUG_SERIAL.print(bytes_written);
      DEBUG_SERIAL.println(" bytes written");
      success = 1;
    }
    delay(1);
  }
}

void loop() {}
